@@ -22,13 +22,13 @@ from airflow.utils.context import Context
 
 class EmptyOperator(BaseOperator):
     """
-    Operator that does literally nothing. It can be used to group tasks in a
-    DAG.
+    Operator that does literally nothing.
 
+    It can be used to group tasks in a DAG.
     The task is evaluated by the scheduler but never processed by the executor.
     """
 
-    ui_color = '#e8f7e4'
+    ui_color = "#e8f7e4"
     inherits_from_empty_operator = True
 
     def execute(self, context: Context):
