@@ -27,15 +27,18 @@ from botocore.exceptions import ClientError
 from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException, AirflowNotFoundException
 from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
+from airflow.providers.amazon.aws.utils.waiter import get_state, waiter
+from airflow.utils.helpers import prune_dict
 
 
 class EmrHook(AwsBaseHook):
     """
-    Interact with Amazon Elastic MapReduce Service.
+    Interact with Amazon Elastic MapReduce Service (EMR).
+    Provide thick wrapper around :external+boto3:py:class:`boto3.client("emr") <EMR.Client>`.
 
     :param emr_conn_id: :ref:`Amazon Elastic MapReduce Connection <howto/connection:emr>`.
         This attribute is only necessary when using
-        the :meth:`~airflow.providers.amazon.aws.hooks.emr.EmrHook.create_job_flow` method.
+        the :meth:`airflow.providers.amazon.aws.hooks.emr.EmrHook.create_job_flow`.
 
     Additional arguments (such as ``aws_conn_id``) may be specified and
     are passed down to the underlying AwsBaseHook.
@@ -59,6 +62,9 @@ class EmrHook(AwsBaseHook):
         Fetch id of EMR cluster with given name and (optional) states.
         Will return only if single id is found.
 
+        .. seealso::
+            - :external+boto3:py:meth:`EMR.Client.list_clusters`
+
         :param emr_cluster_name: Name of a cluster to find
         :param cluster_states: State(s) of cluster to find
         :return: id of the EMR cluster
@@ -83,18 +89,21 @@ class EmrHook(AwsBaseHook):
         """
         Create and start running a new cluster (job flow).
 
+        .. seealso::
+            - :external+boto3:py:meth:`EMR.Client.run_job_flow`
+
         This method uses ``EmrHook.emr_conn_id`` to receive the initial Amazon EMR cluster configuration.
         If ``EmrHook.emr_conn_id`` is empty or the connection does not exist, then an empty initial
         configuration is used.
 
         :param job_flow_overrides: Is used to overwrite the parameters in the initial Amazon EMR configuration
-            cluster. The resulting configuration will be used in the boto3 emr client run_job_flow method.
+            cluster. The resulting configuration will be used in the
+            :external+boto3:py:meth:`EMR.Client.run_job_flow`.
 
         .. seealso::
             - :ref:`Amazon Elastic MapReduce Connection <howto/connection:emr>`
+            - :external+boto3:py:meth:`EMR.Client.run_job_flow`
             - `API RunJobFlow <https://docs.aws.amazon.com/emr/latest/APIReference/API_RunJobFlow.html>`_
-            - `boto3 emr client run_job_flow method <https://boto3.amazonaws.com/v1/documentation/\
-               api/latest/reference/services/emr.html#EMR.Client.run_job_flow>`_.
         """
         config = {}
         if self.emr_conn_id:
@@ -125,16 +134,31 @@ class EmrHook(AwsBaseHook):
         return response
 
     def add_job_flow_steps(
-        self, job_flow_id: str, steps: list[dict] | str | None = None, wait_for_completion: bool = False
+        self,
+        job_flow_id: str,
+        steps: list[dict] | str | None = None,
+        wait_for_completion: bool = False,
+        waiter_delay: int | None = None,
+        waiter_max_attempts: int | None = None,
+        execution_role_arn: str | None = None,
     ) -> list[str]:
         """
         Add new steps to a running cluster.
 
+        .. seealso::
+            - :external+boto3:py:meth:`EMR.Client.add_job_flow_steps`
+
         :param job_flow_id: The id of the job flow to which the steps are being added
         :param steps: A list of the steps to be executed by the job flow
         :param wait_for_completion: If True, wait for the steps to be completed. Default is False
+        :param waiter_delay: The amount of time in seconds to wait between attempts. Default is 5
+        :param waiter_max_attempts: The maximum number of attempts to be made. Default is 100
+        :param execution_role_arn: The ARN of the runtime role for a step on the cluster.
         """
-        response = self.get_conn().add_job_flow_steps(JobFlowId=job_flow_id, Steps=steps)
+        config = {}
+        if execution_role_arn:
+            config["ExecutionRoleArn"] = execution_role_arn
+        response = self.get_conn().add_job_flow_steps(JobFlowId=job_flow_id, Steps=steps, **config)
 
         if response["ResponseMetadata"]["HTTPStatusCode"] != 200:
             raise AirflowException(f"Adding steps failed: {response}")
@@ -146,10 +170,12 @@ class EmrHook(AwsBaseHook):
                 waiter.wait(
                     ClusterId=job_flow_id,
                     StepId=step_id,
-                    WaiterConfig={
-                        "Delay": 5,
-                        "MaxAttempts": 100,
-                    },
+                    WaiterConfig=prune_dict(
+                        {
+                            "Delay": waiter_delay,
+                            "MaxAttempts": waiter_max_attempts,
+                        }
+                    ),
                 )
         return response["StepIds"]
 
@@ -157,6 +183,7 @@ class EmrHook(AwsBaseHook):
         """
         Return failed state for test Amazon Elastic MapReduce Connection (untestable).
 
+
         We need to overwrite this method because this hook is based on
         :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsGenericHook`,
         otherwise it will try to test connection to AWS STS by using the default boto3 credential strategy.
@@ -204,13 +231,14 @@ class EmrHook(AwsBaseHook):
 
 class EmrServerlessHook(AwsBaseHook):
     """
-    Interact with EMR Serverless API.
+    Interact with Amazon EMR Serverless.
+    Provide thin wrapper around :py:class:`boto3.client("emr-serverless") <EMRServerless.Client>`.
 
     Additional arguments (such as ``aws_conn_id``) may be specified and
     are passed down to the underlying AwsBaseHook.
 
     .. seealso::
-        :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
+        - :class:`airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
     """
 
     JOB_INTERMEDIATE_STATES = {"PENDING", "RUNNING", "SCHEDULED", "SUBMITTED"}
@@ -260,39 +288,46 @@ class EmrServerlessHook(AwsBaseHook):
         :param check_interval_seconds: Number of seconds waiter should wait before attempting
             to retry get_state_callable. Defaults to 60 seconds.
         """
-        response = get_state_callable(**get_state_args)
-        state: str = self.get_state(response, parse_response)
-        while state not in desired_state:
-            if state in failure_states:
-                raise AirflowException(f"{object_type.title()} reached failure state {state}.")
-            if countdown >= check_interval_seconds:
-                countdown -= check_interval_seconds
-                self.log.info("Waiting for %s to be %s.", object_type.lower(), action.lower())
-                sleep(check_interval_seconds)
-                state = self.get_state(get_state_callable(**get_state_args), parse_response)
-            else:
-                message = f"{object_type.title()} still not {action.lower()} after the allocated time limit."
-                self.log.error(message)
-                raise RuntimeError(message)
+        warnings.warn(
+            """This method is deprecated.
+            Please use `airflow.providers.amazon.aws.utils.waiter.waiter`.""",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        waiter(
+            get_state_callable=get_state_callable,
+            get_state_args=get_state_args,
+            parse_response=parse_response,
+            desired_state=desired_state,
+            failure_states=failure_states,
+            object_type=object_type,
+            action=action,
+            countdown=countdown,
+            check_interval_seconds=check_interval_seconds,
+        )
 
     def get_state(self, response, keys) -> str:
-        value = response
-        for key in keys:
-            if value is not None:
-                value = value.get(key, None)
-        return value
+        warnings.warn(
+            """This method is deprecated.
+            Please use `airflow.providers.amazon.aws.utils.waiter.get_state`.""",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return get_state(response=response, keys=keys)
 
 
 class EmrContainerHook(AwsBaseHook):
     """
-    Interact with AWS EMR Virtual Cluster to run, poll jobs and return job status
+    Interact with Amazon EMR Containers (Amazon EMR on EKS).
+    Provide thick wrapper around :py:class:`boto3.client("emr-containers") <EMRContainers.Client>`.
+
+    :param virtual_cluster_id: Cluster ID of the EMR on EKS virtual cluster
+
     Additional arguments (such as ``aws_conn_id``) may be specified and
     are passed down to the underlying AwsBaseHook.
 
     .. seealso::
-        :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
-
-    :param virtual_cluster_id: Cluster ID of the EMR on EKS virtual cluster
+        - :class:`airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
     """
 
     INTERMEDIATE_STATES = (
@@ -357,7 +392,9 @@ class EmrContainerHook(AwsBaseHook):
         Submit a job to the EMR Containers API and return the job ID.
         A job run is a unit of work, such as a Spark jar, PySpark script,
         or SparkSQL query, that you submit to Amazon EMR on EKS.
-        See: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr-containers.html#EMRContainers.Client.start_job_run  # noqa: E501
+
+        .. seealso::
+            - :external+boto3:py:meth:`EMRContainers.Client.start_job_run`
 
         :param name: The name of the job run.
         :param execution_role_arn: The IAM role ARN associated with the job run.
@@ -368,7 +405,7 @@ class EmrContainerHook(AwsBaseHook):
         :param client_request_token: The client idempotency token of the job run request.
             Use this if you want to specify a unique ID to prevent two jobs from getting started.
         :param tags: The tags assigned to job runs.
-        :return: Job ID
+        :return: The ID of the job run request.
         """
         params = {
             "name": name,
@@ -398,11 +435,12 @@ class EmrContainerHook(AwsBaseHook):
         """
         Fetch the reason for a job failure (e.g. error message). Returns None or reason string.
 
-        :param job_id: Id of submitted job run
-        :return: str
+        .. seealso::
+            - :external+boto3:py:meth:`EMRContainers.Client.describe_job_run`
+
+        :param job_id: The ID of the job run request.
         """
-        # We absorb any errors if we can't retrieve the job status
-        reason = None
+        reason = None  # We absorb any errors if we can't retrieve the job status
 
         try:
             response = self.conn.describe_job_run(
@@ -422,9 +460,11 @@ class EmrContainerHook(AwsBaseHook):
     def check_query_status(self, job_id: str) -> str | None:
         """
         Fetch the status of submitted job run. Returns None or one of valid query states.
-        See: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/emr-containers.html#EMRContainers.Client.describe_job_run  # noqa: E501
-        :param job_id: Id of submitted job run
-        :return: str
+
+        .. seealso::
+            - :external+boto3:py:meth:`EMRContainers.Client.describe_job_run`
+
+        :param job_id: The ID of the job run request.
         """
         try:
             response = self.conn.describe_job_run(
@@ -451,11 +491,10 @@ class EmrContainerHook(AwsBaseHook):
         Poll the status of submitted job run until query state reaches final state.
         Returns one of the final states.
 
-        :param job_id: Id of submitted job run
+        :param job_id: The ID of the job run request.
         :param max_tries: Deprecated - Use max_polling_attempts instead
         :param poll_interval: Time (in seconds) to wait between calls to check query status on EMR
         :param max_polling_attempts: Number of times to poll for query state before function exits
-        :return: str
         """
         if max_tries:
             warnings.warn(
@@ -495,8 +534,10 @@ class EmrContainerHook(AwsBaseHook):
         """
         Cancel the submitted job_run
 
-        :param job_id: Id of submitted job_run
-        :return: dict
+        .. seealso::
+            - :external+boto3:py:meth:`EMRContainers.Client.cancel_job_run`
+
+        :param job_id: The ID of the job run to cancel.
         """
         return self.conn.cancel_job_run(
             virtualClusterId=self.virtual_cluster_id,
