@@ -15,25 +15,27 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """
 This module contains a hook (AwsLogsHook) with some very basic
 functionality for interacting with AWS CloudWatch.
 """
-from typing import Dict, Generator, Optional
+from __future__ import annotations
+
+from typing import Generator
 
 from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
 
 
 class AwsLogsHook(AwsBaseHook):
     """
-    Interact with AWS CloudWatch Logs
+    Interact with Amazon CloudWatch Logs.
+    Provide thin wrapper around :external+boto3:py:class:`boto3.client("logs") <CloudWatchLogs.Client>`.
 
     Additional arguments (such as ``aws_conn_id``) may be specified and
     are passed down to the underlying AwsBaseHook.
 
     .. seealso::
-        :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
+        - :class:`airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
     """
 
     def __init__(self, *args, **kwargs) -> None:
@@ -52,6 +54,9 @@ class AwsLogsHook(AwsBaseHook):
         A generator for log items in a single stream. This will yield all the
         items that are available at the current moment.
 
+        .. seealso::
+            - :external+boto3:py:meth:`CloudWatchLogs.Client.get_log_events`
+
         :param log_group: The name of the log group.
         :param log_stream_name: The name of the specific stream.
         :param start_time: The time stamp value to start reading the logs from (default: 0).
@@ -59,7 +64,6 @@ class AwsLogsHook(AwsBaseHook):
             This is for when there are multiple entries at the same timestamp.
         :param start_from_head: whether to start from the beginning (True) of the log or
             at the end of the log (False).
-        :rtype: dict
         :return: | A CloudWatch log event with the following key-value pairs:
                  |   'timestamp' (int): The time in milliseconds of the event.
                  |   'message' (str): The log event data.
@@ -68,7 +72,7 @@ class AwsLogsHook(AwsBaseHook):
         next_token = None
         while True:
             if next_token is not None:
-                token_arg: Optional[Dict[str, str]] = {'nextToken': next_token}
+                token_arg: dict[str, str] | None = {"nextToken": next_token}
             else:
                 token_arg = {}
 
@@ -80,7 +84,7 @@ class AwsLogsHook(AwsBaseHook):
                 **token_arg,
             )
 
-            events = response['events']
+            events = response["events"]
             event_count = len(events)
 
             if event_count > skip:
@@ -92,7 +96,7 @@ class AwsLogsHook(AwsBaseHook):
 
             yield from events
 
-            if next_token != response['nextForwardToken']:
-                next_token = response['nextForwardToken']
+            if next_token != response["nextForwardToken"]:
+                next_token = response["nextForwardToken"]
             else:
                 return
