@@ -21,18 +21,26 @@ from __future__ import annotations
 from tempfile import NamedTemporaryFile
 from typing import IO, Any
 
-from google.ads.googleads.client import GoogleAdsClient
-from google.ads.googleads.errors import GoogleAdsException
-from google.ads.googleads.v10.services.services.customer_service import CustomerServiceClient
-from google.ads.googleads.v10.services.services.google_ads_service import GoogleAdsServiceClient
-from google.ads.googleads.v10.services.types.google_ads_service import GoogleAdsRow, SearchGoogleAdsRequest
-from google.api_core.page_iterator import GRPCIterator
 from google.auth.exceptions import GoogleAuthError
 
 from airflow import AirflowException
 from airflow.compat.functools import cached_property
 from airflow.hooks.base import BaseHook
 from airflow.providers.google.common.hooks.base_google import get_field
+from airflow.providers.google_vendor.googleads.client import GoogleAdsClient
+from airflow.providers.google_vendor.googleads.errors import GoogleAdsException
+from airflow.providers.google_vendor.googleads.v12.services.services.customer_service import (
+    CustomerServiceClient,
+)
+from airflow.providers.google_vendor.googleads.v12.services.services.google_ads_service import (
+    GoogleAdsServiceClient,
+)
+from airflow.providers.google_vendor.googleads.v12.services.services.google_ads_service.pagers import (
+    SearchPager,
+)
+from airflow.providers.google_vendor.googleads.v12.services.types.google_ads_service import (
+    GoogleAdsRow,
+)
 
 
 class GoogleAdsHook(BaseHook):
@@ -73,7 +81,7 @@ class GoogleAdsHook(BaseHook):
     :return: list of Google Ads Row object(s)
     """
 
-    default_api_version = "v10"
+    default_api_version = "v12"
 
     def __init__(
         self,
@@ -223,19 +231,14 @@ class GoogleAdsHook(BaseHook):
 
         iterators = []
         for client_id in client_ids:
-            request: SearchGoogleAdsRequest = self._get_client.get_type("SearchGoogleAdsRequest")
-            request.customer_id = client_id
-            request.query = query
-            request.page_size = page_size
-
-            iterator = service.search(request=request)
+            iterator = service.search(request=dict(customer_id=client_id, query=query, page_size=page_size))
             iterators.append(iterator)
 
         self.log.info("Fetched Google Ads Iterators")
 
         return self._extract_rows(iterators)
 
-    def _extract_rows(self, iterators: list[GRPCIterator]) -> list[GoogleAdsRow]:
+    def _extract_rows(self, iterators: list[SearchPager]) -> list[GoogleAdsRow]:
         """
         Convert Google Page Iterator (GRPCIterator) objects to Google Ads Rows
 
