@@ -17,6 +17,7 @@
 """This module contains Google Dataplex hook."""
 from __future__ import annotations
 
+import warnings
 from typing import Any, Sequence
 
 from google.api_core.client_options import ClientOptions
@@ -59,6 +60,10 @@ class DataplexHook(GoogleBaseHook):
         delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
     ) -> None:
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         super().__init__(
             gcp_conn_id=gcp_conn_id,
             delegate_to=delegate_to,
