@@ -26,6 +26,7 @@
 """
 from __future__ import annotations
 
+import warnings
 from typing import Sequence
 
 from google.api_core.client_options import ClientOptions
@@ -49,6 +50,10 @@ class BatchPredictionJobHook(GoogleBaseHook):
         delegate_to: str | None = None,
         impersonation_chain: str | Sequence[str] | None = None,
     ) -> None:
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         super().__init__(
             gcp_conn_id=gcp_conn_id,
             delegate_to=delegate_to,
