@@ -15,10 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
 """This module contains a Google Cloud Vertex AI hook."""
+from __future__ import annotations
 
-from typing import Dict, Optional, Sequence, Tuple, Union
+from typing import Sequence
 
 from google.api_core.client_options import ClientOptions
 from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
@@ -41,10 +41,10 @@ from airflow.providers.google.common.hooks.base_google import GoogleBaseHook
 class DatasetHook(GoogleBaseHook):
     """Hook for Google Cloud Vertex AI Dataset APIs."""
 
-    def get_dataset_service_client(self, region: Optional[str] = None) -> DatasetServiceClient:
+    def get_dataset_service_client(self, region: str | None = None) -> DatasetServiceClient:
         """Returns DatasetServiceClient."""
-        if region and region != 'global':
-            client_options = ClientOptions(api_endpoint=f'{region}-aiplatform.googleapis.com:443')
+        if region and region != "global":
+            client_options = ClientOptions(api_endpoint=f"{region}-aiplatform.googleapis.com:443")
         else:
             client_options = ClientOptions()
 
@@ -52,7 +52,7 @@ class DatasetHook(GoogleBaseHook):
             credentials=self.get_credentials(), client_info=CLIENT_INFO, client_options=client_options
         )
 
-    def wait_for_operation(self, operation: Operation, timeout: Optional[float] = None):
+    def wait_for_operation(self, operation: Operation, timeout: float | None = None):
         """Waits for long-lasting operation to complete."""
         try:
             return operation.result(timeout=timeout)
@@ -61,7 +61,7 @@ class DatasetHook(GoogleBaseHook):
             raise AirflowException(error)
 
     @staticmethod
-    def extract_dataset_id(obj: Dict) -> str:
+    def extract_dataset_id(obj: dict) -> str:
         """Returns unique id of the dataset."""
         return obj["name"].rpartition("/")[-1]
 
@@ -70,10 +70,10 @@ class DatasetHook(GoogleBaseHook):
         self,
         project_id: str,
         region: str,
-        dataset: Union[Dataset, Dict],
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        dataset: Dataset | dict,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Operation:
         """
         Creates a Dataset.
@@ -90,8 +90,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.create_dataset(
             request={
-                'parent': parent,
-                'dataset': dataset,
+                "parent": parent,
+                "dataset": dataset,
             },
             retry=retry,
             timeout=timeout,
@@ -105,9 +105,9 @@ class DatasetHook(GoogleBaseHook):
         project_id: str,
         region: str,
         dataset: str,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Operation:
         """
         Deletes a Dataset.
@@ -124,7 +124,7 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.delete_dataset(
             request={
-                'name': name,
+                "name": name,
             },
             retry=retry,
             timeout=timeout,
@@ -138,10 +138,10 @@ class DatasetHook(GoogleBaseHook):
         project_id: str,
         region: str,
         dataset: str,
-        export_config: Union[ExportDataConfig, Dict],
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        export_config: ExportDataConfig | dict,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Operation:
         """
         Exports data from a Dataset.
@@ -159,8 +159,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.export_data(
             request={
-                'name': name,
-                'export_config': export_config,
+                "name": name,
+                "export_config": export_config,
             },
             retry=retry,
             timeout=timeout,
@@ -175,10 +175,10 @@ class DatasetHook(GoogleBaseHook):
         region: str,
         dataset: str,
         annotation_spec: str,
-        read_mask: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        read_mask: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> AnnotationSpec:
         """
         Gets an AnnotationSpec.
@@ -197,8 +197,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.get_annotation_spec(
             request={
-                'name': name,
-                'read_mask': read_mask,
+                "name": name,
+                "read_mask": read_mask,
             },
             retry=retry,
             timeout=timeout,
@@ -212,10 +212,10 @@ class DatasetHook(GoogleBaseHook):
         project_id: str,
         region: str,
         dataset: str,
-        read_mask: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        read_mask: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Dataset:
         """
         Gets a Dataset.
@@ -233,8 +233,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.get_dataset(
             request={
-                'name': name,
-                'read_mask': read_mask,
+                "name": name,
+                "read_mask": read_mask,
             },
             retry=retry,
             timeout=timeout,
@@ -249,9 +249,9 @@ class DatasetHook(GoogleBaseHook):
         region: str,
         dataset: str,
         import_configs: Sequence[ImportDataConfig],
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Operation:
         """
         Imports data into a Dataset.
@@ -270,8 +270,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.import_data(
             request={
-                'name': name,
-                'import_configs': import_configs,
+                "name": name,
+                "import_configs": import_configs,
             },
             retry=retry,
             timeout=timeout,
@@ -286,14 +286,14 @@ class DatasetHook(GoogleBaseHook):
         region: str,
         dataset: str,
         data_item: str,
-        filter: Optional[str] = None,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        read_mask: Optional[str] = None,
-        order_by: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        filter: str | None = None,
+        page_size: int | None = None,
+        page_token: str | None = None,
+        read_mask: str | None = None,
+        order_by: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> ListAnnotationsPager:
         """
         Lists Annotations belongs to a data item
@@ -317,12 +317,12 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.list_annotations(
             request={
-                'parent': parent,
-                'filter': filter,
-                'page_size': page_size,
-                'page_token': page_token,
-                'read_mask': read_mask,
-                'order_by': order_by,
+                "parent": parent,
+                "filter": filter,
+                "page_size": page_size,
+                "page_token": page_token,
+                "read_mask": read_mask,
+                "order_by": order_by,
             },
             retry=retry,
             timeout=timeout,
@@ -336,14 +336,14 @@ class DatasetHook(GoogleBaseHook):
         project_id: str,
         region: str,
         dataset: str,
-        filter: Optional[str] = None,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        read_mask: Optional[str] = None,
-        order_by: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        filter: str | None = None,
+        page_size: int | None = None,
+        page_token: str | None = None,
+        read_mask: str | None = None,
+        order_by: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> ListDataItemsPager:
         """
         Lists DataItems in a Dataset.
@@ -366,12 +366,12 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.list_data_items(
             request={
-                'parent': parent,
-                'filter': filter,
-                'page_size': page_size,
-                'page_token': page_token,
-                'read_mask': read_mask,
-                'order_by': order_by,
+                "parent": parent,
+                "filter": filter,
+                "page_size": page_size,
+                "page_token": page_token,
+                "read_mask": read_mask,
+                "order_by": order_by,
             },
             retry=retry,
             timeout=timeout,
@@ -384,14 +384,14 @@ class DatasetHook(GoogleBaseHook):
         self,
         project_id: str,
         region: str,
-        filter: Optional[str] = None,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        read_mask: Optional[str] = None,
-        order_by: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        filter: str | None = None,
+        page_size: int | None = None,
+        page_token: str | None = None,
+        read_mask: str | None = None,
+        order_by: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> ListDatasetsPager:
         """
         Lists Datasets in a Location.
@@ -413,12 +413,12 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.list_datasets(
             request={
-                'parent': parent,
-                'filter': filter,
-                'page_size': page_size,
-                'page_token': page_token,
-                'read_mask': read_mask,
-                'order_by': order_by,
+                "parent": parent,
+                "filter": filter,
+                "page_size": page_size,
+                "page_token": page_token,
+                "read_mask": read_mask,
+                "order_by": order_by,
             },
             retry=retry,
             timeout=timeout,
@@ -431,11 +431,11 @@ class DatasetHook(GoogleBaseHook):
         project_id: str,
         region: str,
         dataset_id: str,
-        dataset: Union[Dataset, Dict],
-        update_mask: Union[FieldMask, Dict],
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        dataset: Dataset | dict,
+        update_mask: FieldMask | dict,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
     ) -> Dataset:
         """
         Updates a Dataset.
@@ -454,8 +454,8 @@ class DatasetHook(GoogleBaseHook):
 
         result = client.update_dataset(
             request={
-                'dataset': dataset,
-                'update_mask': update_mask,
+                "dataset": dataset,
+                "update_mask": update_mask,
             },
             retry=retry,
             timeout=timeout,
