@@ -18,12 +18,12 @@
 """This module contains Google Cloud Transfer operators."""
 from __future__ import annotations
 
+import warnings
 from copy import deepcopy
 from datetime import date, time
 from typing import TYPE_CHECKING, Sequence
 
 from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
 from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
 from airflow.providers.google.cloud.hooks.cloud_storage_transfer_service import (
     ACCESS_KEY_ID,
@@ -60,6 +60,7 @@ from airflow.providers.google.cloud.links.cloud_storage_transfer import (
     CloudStorageTransferJobLink,
     CloudStorageTransferListLink,
 )
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 from airflow.providers.google.cloud.utils.helpers import normalize_directory_path
 
 if TYPE_CHECKING:
@@ -173,7 +174,7 @@ class TransferJobValidator:
             self._verify_data_source()
 
 
-class CloudDataTransferServiceCreateJobOperator(BaseOperator):
+class CloudDataTransferServiceCreateJobOperator(GoogleCloudBaseOperator):
     """
     Creates a transfer job that runs periodically.
 
@@ -268,7 +269,7 @@ class CloudDataTransferServiceCreateJobOperator(BaseOperator):
         return result
 
 
-class CloudDataTransferServiceUpdateJobOperator(BaseOperator):
+class CloudDataTransferServiceUpdateJobOperator(GoogleCloudBaseOperator):
     """
     Updates a transfer job that runs periodically.
 
@@ -358,7 +359,7 @@ class CloudDataTransferServiceUpdateJobOperator(BaseOperator):
         return hook.update_transfer_job(job_name=self.job_name, body=self.body)
 
 
-class CloudDataTransferServiceDeleteJobOperator(BaseOperator):
+class CloudDataTransferServiceDeleteJobOperator(GoogleCloudBaseOperator):
     """
     Delete a transfer job. This is a soft delete. After a transfer job is
     deleted, the job and all the transfer executions are subject to garbage
@@ -427,7 +428,7 @@ class CloudDataTransferServiceDeleteJobOperator(BaseOperator):
         hook.delete_transfer_job(job_name=self.job_name, project_id=self.project_id)
 
 
-class CloudDataTransferServiceGetOperationOperator(BaseOperator):
+class CloudDataTransferServiceGetOperationOperator(GoogleCloudBaseOperator):
     """
     Gets the latest state of a long-running operation in Google Storage Transfer
     Service.
@@ -501,7 +502,7 @@ class CloudDataTransferServiceGetOperationOperator(BaseOperator):
         return operation
 
 
-class CloudDataTransferServiceListOperationsOperator(BaseOperator):
+class CloudDataTransferServiceListOperationsOperator(GoogleCloudBaseOperator):
     """
     Lists long-running operations in Google Storage Transfer
     Service that match the specified filter.
@@ -584,7 +585,7 @@ class CloudDataTransferServiceListOperationsOperator(BaseOperator):
         return operations_list
 
 
-class CloudDataTransferServicePauseOperationOperator(BaseOperator):
+class CloudDataTransferServicePauseOperationOperator(GoogleCloudBaseOperator):
     """
     Pauses a transfer operation in Google Storage Transfer Service.
 
@@ -643,7 +644,7 @@ class CloudDataTransferServicePauseOperationOperator(BaseOperator):
         hook.pause_transfer_operation(operation_name=self.operation_name)
 
 
-class CloudDataTransferServiceResumeOperationOperator(BaseOperator):
+class CloudDataTransferServiceResumeOperationOperator(GoogleCloudBaseOperator):
     """
     Resumes a transfer operation in Google Storage Transfer Service.
 
@@ -702,7 +703,7 @@ class CloudDataTransferServiceResumeOperationOperator(BaseOperator):
         hook.resume_transfer_operation(operation_name=self.operation_name)
 
 
-class CloudDataTransferServiceCancelOperationOperator(BaseOperator):
+class CloudDataTransferServiceCancelOperationOperator(GoogleCloudBaseOperator):
     """
     Cancels a transfer operation in Google Storage Transfer Service.
 
@@ -762,7 +763,7 @@ class CloudDataTransferServiceCancelOperationOperator(BaseOperator):
         hook.cancel_transfer_operation(operation_name=self.operation_name)
 
 
-class CloudDataTransferServiceS3ToGCSOperator(BaseOperator):
+class CloudDataTransferServiceS3ToGCSOperator(GoogleCloudBaseOperator):
     """
     Synchronizes an S3 bucket with a Google Cloud Storage bucket using the
     Google Cloud Storage Transfer Service.
@@ -868,6 +869,10 @@ class CloudDataTransferServiceS3ToGCSOperator(BaseOperator):
         self.project_id = project_id
         self.aws_conn_id = aws_conn_id
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.description = description
         self.schedule = schedule
@@ -931,7 +936,7 @@ class CloudDataTransferServiceS3ToGCSOperator(BaseOperator):
         return body
 
 
-class CloudDataTransferServiceGCSToGCSOperator(BaseOperator):
+class CloudDataTransferServiceGCSToGCSOperator(GoogleCloudBaseOperator):
     """
     Copies objects from a bucket to another using the Google Cloud Storage Transfer Service.
 
@@ -1038,6 +1043,10 @@ class CloudDataTransferServiceGCSToGCSOperator(BaseOperator):
         self.destination_path = destination_path
         self.project_id = project_id
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.description = description
         self.schedule = schedule
