@@ -15,28 +15,41 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
 """This module contains Google Kubernetes Engine operators."""
+from __future__ import annotations
 
-import os
-import tempfile
 import warnings
-from typing import TYPE_CHECKING, Dict, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Sequence
 
+from google.api_core.exceptions import AlreadyExists
 from google.cloud.container_v1.types import Cluster
+from kubernetes.client.models import V1Pod
 
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
-from airflow.providers.cncf.kubernetes.operators.kubernetes_pod import KubernetesPodOperator
-from airflow.providers.google.cloud.hooks.kubernetes_engine import GKEHook
-from airflow.providers.google.common.hooks.base_google import GoogleBaseHook
-from airflow.utils.process_utils import execute_in_subprocess, patch_environ
+
+try:
+    from airflow.providers.cncf.kubernetes.operators.pod import KubernetesPodOperator
+except ImportError:
+    # preserve backward compatibility for older versions of cncf.kubernetes provider
+    from airflow.providers.cncf.kubernetes.operators.kubernetes_pod import KubernetesPodOperator
+from airflow.providers.google.cloud.hooks.kubernetes_engine import GKEHook, GKEPodHook
+from airflow.providers.google.cloud.links.kubernetes_engine import (
+    KubernetesEngineClusterLink,
+    KubernetesEnginePodLink,
+)
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
+from airflow.providers.google.cloud.triggers.kubernetes_engine import GKEOperationTrigger, GKEStartPodTrigger
+from airflow.utils.timezone import utcnow
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-class GKEDeleteClusterOperator(BaseOperator):
+KUBE_CONFIG_ENV_VAR = "KUBECONFIG"
+
+
+class GKEDeleteClusterOperator(GoogleCloudBaseOperator):
     """
     Deletes the cluster, including the Kubernetes endpoint and all worker nodes.
 
@@ -73,15 +86,17 @@ class GKEDeleteClusterOperator(BaseOperator):
         If set as a sequence, the identities from the list must grant
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
+    :param deferrable: Run operator in the deferrable mode.
+    :param poll_interval: Interval size which defines how often operation status is checked.
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'gcp_conn_id',
-        'name',
-        'location',
-        'api_version',
-        'impersonation_chain',
+        "project_id",
+        "gcp_conn_id",
+        "name",
+        "location",
+        "api_version",
+        "impersonation_chain",
     )
 
     def __init__(
@@ -89,10 +104,12 @@ class GKEDeleteClusterOperator(BaseOperator):
         *,
         name: str,
         location: str,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        api_version: str = 'v2',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        api_version: str = "v2",
+        impersonation_chain: str | Sequence[str] | None = None,
+        deferrable: bool = False,
+        poll_interval: int = 10,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -103,24 +120,69 @@ class GKEDeleteClusterOperator(BaseOperator):
         self.api_version = api_version
         self.name = name
         self.impersonation_chain = impersonation_chain
+        self.deferrable = deferrable
+        self.poll_interval = poll_interval
         self._check_input()
 
+        self._hook: GKEHook | None = None
+
     def _check_input(self) -> None:
         if not all([self.project_id, self.name, self.location]):
-            self.log.error('One of (project_id, name, location) is missing or incorrect')
-            raise AirflowException('Operator has incorrect or missing input.')
+            self.log.error("One of (project_id, name, location) is missing or incorrect")
+            raise AirflowException("Operator has incorrect or missing input.")
 
-    def execute(self, context: 'Context') -> Optional[str]:
-        hook = GKEHook(
-            gcp_conn_id=self.gcp_conn_id,
-            location=self.location,
-            impersonation_chain=self.impersonation_chain,
+    def execute(self, context: Context) -> str | None:
+        hook = self._get_hook()
+
+        wait_to_complete = not self.deferrable
+        operation = hook.delete_cluster(
+            name=self.name,
+            project_id=self.project_id,
+            wait_to_complete=wait_to_complete,
         )
-        delete_result = hook.delete_cluster(name=self.name, project_id=self.project_id)
-        return delete_result
+
+        if self.deferrable and operation is not None:
+            self.defer(
+                trigger=GKEOperationTrigger(
+                    operation_name=operation.name,
+                    project_id=self.project_id,
+                    location=self.location,
+                    gcp_conn_id=self.gcp_conn_id,
+                    impersonation_chain=self.impersonation_chain,
+                    poll_interval=self.poll_interval,
+                ),
+                method_name="execute_complete",
+            )
+
+        return operation.self_link if operation is not None else None
+
+    def execute_complete(self, context: Context, event: dict) -> str:
+        """Method to be executed after trigger job is done."""
+        status = event["status"]
+        message = event["message"]
+
+        if status == "failed" or status == "error":
+            self.log.exception("Trigger ended with one of the failed statuses.")
+            raise AirflowException(message)
+
+        self.log.info(message)
+        operation = self._get_hook().get_operation(
+            operation_name=event["operation_name"],
+        )
+        return operation.self_link
+
+    def _get_hook(self) -> GKEHook:
+        if self._hook is None:
+            self._hook = GKEHook(
+                gcp_conn_id=self.gcp_conn_id,
+                location=self.location,
+                impersonation_chain=self.impersonation_chain,
+            )
+
+        return self._hook
 
 
-class GKECreateClusterOperator(BaseOperator):
+class GKECreateClusterOperator(GoogleCloudBaseOperator):
     """
     Create a Google Kubernetes Engine Cluster of specified dimensions
     The operator will wait until the cluster is created.
@@ -169,26 +231,31 @@ class GKECreateClusterOperator(BaseOperator):
         If set as a sequence, the identities from the list must grant
         Service Account Token Creator IAM role to the directly preceding identity, with first
         account from the list granting this role to the originating account (templated).
+    :param deferrable: Run operator in the deferrable mode.
+    :param poll_interval: Interval size which defines how often operation status is checked.
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'gcp_conn_id',
-        'location',
-        'api_version',
-        'body',
-        'impersonation_chain',
+        "project_id",
+        "gcp_conn_id",
+        "location",
+        "api_version",
+        "body",
+        "impersonation_chain",
     )
+    operator_extra_links = (KubernetesEngineClusterLink(),)
 
     def __init__(
         self,
         *,
         location: str,
-        body: Optional[Union[Dict, Cluster]],
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        api_version: str = 'v2',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        body: dict | Cluster,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        api_version: str = "v2",
+        impersonation_chain: str | Sequence[str] | None = None,
+        poll_interval: int = 10,
+        deferrable: bool = False,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -199,12 +266,16 @@ class GKECreateClusterOperator(BaseOperator):
         self.api_version = api_version
         self.body = body
         self.impersonation_chain = impersonation_chain
+        self.poll_interval = poll_interval
+        self.deferrable = deferrable
         self._check_input()
 
+        self._hook: GKEHook | None = None
+
     def _check_input(self) -> None:
         if (
             not all([self.project_id, self.location, self.body])
-            or (isinstance(self.body, dict) and not ("name" in self.body))
+            or (isinstance(self.body, dict) and "name" not in self.body)
             or (
                 isinstance(self.body, dict)
                 and ("initial_node_count" not in self.body and "node_pools" not in self.body)
@@ -232,23 +303,67 @@ class GKECreateClusterOperator(BaseOperator):
             self.log.error("Only one of body['initial_node_count']) and body['node_pools'] may be specified")
             raise AirflowException("Operator has incorrect or missing input.")
 
-    def execute(self, context: 'Context') -> str:
-        hook = GKEHook(
-            gcp_conn_id=self.gcp_conn_id,
-            location=self.location,
-            impersonation_chain=self.impersonation_chain,
+    def execute(self, context: Context) -> str:
+        hook = self._get_hook()
+        try:
+            wait_to_complete = not self.deferrable
+            operation = hook.create_cluster(
+                cluster=self.body,
+                project_id=self.project_id,
+                wait_to_complete=wait_to_complete,
+            )
+
+            KubernetesEngineClusterLink.persist(context=context, task_instance=self, cluster=self.body)
+
+            if self.deferrable:
+                self.defer(
+                    trigger=GKEOperationTrigger(
+                        operation_name=operation.name,
+                        project_id=self.project_id,
+                        location=self.location,
+                        gcp_conn_id=self.gcp_conn_id,
+                        impersonation_chain=self.impersonation_chain,
+                        poll_interval=self.poll_interval,
+                    ),
+                    method_name="execute_complete",
+                )
+
+            return operation.target_link
+
+        except AlreadyExists as error:
+            self.log.info("Assuming Success: %s", error.message)
+            name = self.body.name if isinstance(self.body, Cluster) else self.body["name"]
+            return hook.get_cluster(name=name, project_id=self.project_id).self_link
+
+    def execute_complete(self, context: Context, event: dict) -> str:
+        status = event["status"]
+        message = event["message"]
+
+        if status == "failed" or status == "error":
+            self.log.exception("Trigger ended with one of the failed statuses.")
+            raise AirflowException(message)
+
+        self.log.info(message)
+        operation = self._get_hook().get_operation(
+            operation_name=event["operation_name"],
         )
-        create_op = hook.create_cluster(cluster=self.body, project_id=self.project_id)
-        return create_op
+        return operation.target_link
 
+    def _get_hook(self) -> GKEHook:
+        if self._hook is None:
+            self._hook = GKEHook(
+                gcp_conn_id=self.gcp_conn_id,
+                location=self.location,
+                impersonation_chain=self.impersonation_chain,
+            )
 
-KUBE_CONFIG_ENV_VAR = "KUBECONFIG"
+        return self._hook
 
 
 class GKEStartPodOperator(KubernetesPodOperator):
     """
     Executes a task in a Kubernetes pod in the specified Google Kubernetes
-    Engine cluster
+    Engine cluster.
 
     This Operator assumes that the system has gcloud installed and has configured a
     connection id with a service account.
@@ -271,7 +386,7 @@ class GKEStartPodOperator(KubernetesPodOperator):
         should be spawned in
     :param use_internal_ip: Use the internal IP address as the endpoint.
     :param project_id: The Google Developers Console project id
-    :param gcp_conn_id: The google cloud connection id to use. This allows for
+    :param gcp_conn_id: The Google cloud connection id to use. This allows for
         users to specify a service account.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or list of accounts required to get the access_token
@@ -284,25 +399,27 @@ class GKEStartPodOperator(KubernetesPodOperator):
     :param regional: The location param is region name.
     :param is_delete_operator_pod: What to do when the pod reaches its final
         state, or the execution is interrupted. If True, delete the
-        pod; if False, leave the pod.  Current default is False, but this will be
+        pod; if False, leave the pod. Current default is False, but this will be
         changed in the next major release of this provider.
+    :param deferrable: Run operator in the deferrable mode.
     """
 
     template_fields: Sequence[str] = tuple(
-        {'project_id', 'location', 'cluster_name'} | set(KubernetesPodOperator.template_fields)
+        {"project_id", "location", "cluster_name"} | set(KubernetesPodOperator.template_fields)
     )
+    operator_extra_links = (KubernetesEnginePodLink(),)
 
     def __init__(
         self,
         *,
         location: str,
         cluster_name: str,
-        use_internal_ip: bool = False,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        regional: bool = False,
-        is_delete_operator_pod: Optional[bool] = None,
+        use_internal_ip: bool | None = None,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
+        regional: bool | None = None,
+        is_delete_operator_pod: bool | None = None,
         **kwargs,
     ) -> None:
         if is_delete_operator_pod is None:
@@ -316,14 +433,34 @@ class GKEStartPodOperator(KubernetesPodOperator):
             )
             is_delete_operator_pod = False
 
+        if use_internal_ip is not None:
+            warnings.warn(
+                f"You have set parameter use_internal_ip in class {self.__class__.__name__}. "
+                "In current implementation of the operator the parameter is not used and will "
+                "be deleted in future.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if regional is not None:
+            warnings.warn(
+                f"You have set parameter regional in class {self.__class__.__name__}. "
+                "In current implementation of the operator the parameter is not used and will "
+                "be deleted in future.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
         super().__init__(is_delete_operator_pod=is_delete_operator_pod, **kwargs)
         self.project_id = project_id
         self.location = location
         self.cluster_name = cluster_name
         self.gcp_conn_id = gcp_conn_id
-        self.use_internal_ip = use_internal_ip
         self.impersonation_chain = impersonation_chain
-        self.regional = regional
+
+        self.pod: V1Pod | None = None
+        self._ssl_ca_cert: str | None = None
+        self._cluster_url: str | None = None
 
         if self.gcp_conn_id is None:
             raise AirflowException(
@@ -336,62 +473,78 @@ class GKEStartPodOperator(KubernetesPodOperator):
         if self.config_file:
             raise AirflowException("config_file is not an allowed parameter for the GKEStartPodOperator.")
 
-    def execute(self, context: 'Context') -> Optional[str]:
-        hook = GoogleBaseHook(gcp_conn_id=self.gcp_conn_id)
-        self.project_id = self.project_id or hook.project_id
+    @staticmethod
+    def get_gke_config_file():
+        warnings.warn(
+            "The `get_gke_config_file` method is deprecated, "
+            "please use `fetch_cluster_info` instead to get the cluster info for connecting to it.",
+            DeprecationWarning,
+            stacklevel=1,
+        )
 
-        if not self.project_id:
-            raise AirflowException(
-                "The project id must be passed either as "
-                "keyword project_id parameter or as project_id extra "
-                "in Google Cloud connection definition. Both are not set!"
+    @cached_property
+    def cluster_hook(self) -> GKEHook:
+        return GKEHook(
+            gcp_conn_id=self.gcp_conn_id,
+            location=self.location,
+            impersonation_chain=self.impersonation_chain,
+        )
+
+    @cached_property
+    def hook(self) -> GKEPodHook:  # type: ignore[override]
+        if self._cluster_url is None or self._ssl_ca_cert is None:
+            raise AttributeError(
+                "Cluster url and ssl_ca_cert should be defined before using self.hook method. "
+                "Try to use self.get_kube_creds method",
             )
 
-        # Write config to a temp file and set the environment variable to point to it.
-        # This is to avoid race conditions of reading/writing a single file
-        with tempfile.NamedTemporaryFile() as conf_file, patch_environ(
-            {KUBE_CONFIG_ENV_VAR: conf_file.name}
-        ), hook.provide_authorized_gcloud():
-            # Attempt to get/update credentials
-            # We call gcloud directly instead of using google-cloud-python api
-            # because there is no way to write kubernetes config to a file, which is
-            # required by KubernetesPodOperator.
-            # The gcloud command looks at the env variable `KUBECONFIG` for where to save
-            # the kubernetes config file.
-            cmd = [
-                "gcloud",
-                "container",
-                "clusters",
-                "get-credentials",
-                self.cluster_name,
-                "--project",
-                self.project_id,
-            ]
-            if self.impersonation_chain:
-                if isinstance(self.impersonation_chain, str):
-                    impersonation_account = self.impersonation_chain
-                elif len(self.impersonation_chain) == 1:
-                    impersonation_account = self.impersonation_chain[0]
-                else:
-                    raise AirflowException(
-                        "Chained list of accounts is not supported, please specify only one service account"
-                    )
+        hook = GKEPodHook(
+            cluster_url=self._cluster_url,
+            ssl_ca_cert=self._ssl_ca_cert,
+        )
+        return hook
 
-                cmd.extend(
-                    [
-                        '--impersonate-service-account',
-                        impersonation_account,
-                    ]
-                )
-            if self.regional:
-                cmd.append('--region')
-            else:
-                cmd.append('--zone')
-            cmd.append(self.location)
-            if self.use_internal_ip:
-                cmd.append('--internal-ip')
-            execute_in_subprocess(cmd)
+    def execute(self, context: Context):
+        """Executes process of creating pod and executing provided command inside it."""
+        self.fetch_cluster_info()
+        return super().execute(context)
 
-            # Tell `KubernetesPodOperator` where the config file is located
-            self.config_file = os.environ[KUBE_CONFIG_ENV_VAR]
-            return super().execute(context)
+    def fetch_cluster_info(self) -> tuple[str, str | None]:
+        """Fetches cluster info for connecting to it."""
+        cluster = self.cluster_hook.get_cluster(
+            name=self.cluster_name,
+            project_id=self.project_id,
+        )
+
+        self._cluster_url = f"https://{cluster.endpoint}"
+        self._ssl_ca_cert = cluster.master_auth.cluster_ca_certificate
+        return self._cluster_url, self._ssl_ca_cert
+
+    def invoke_defer_method(self):
+        """Method to easily redefine triggers which are being used in child classes."""
+        trigger_start_time = utcnow()
+        self.defer(
+            trigger=GKEStartPodTrigger(
+                pod_name=self.pod.metadata.name,
+                pod_namespace=self.pod.metadata.namespace,
+                trigger_start_time=trigger_start_time,
+                cluster_url=self._cluster_url,
+                ssl_ca_cert=self._ssl_ca_cert,
+                get_logs=self.get_logs,
+                startup_timeout=self.startup_timeout_seconds,
+                cluster_context=self.cluster_context,
+                poll_interval=self.poll_interval,
+                in_cluster=self.in_cluster,
+                should_delete_pod=self.is_delete_operator_pod,
+                base_container_name=self.base_container_name,
+            ),
+            method_name="execute_complete",
+            kwargs={"cluster_url": self._cluster_url, "ssl_ca_cert": self._ssl_ca_cert},
+        )
+
+    def execute_complete(self, context: Context, event: dict, **kwargs):
+        # It is required for hook to be initialized
+        self._cluster_url = kwargs["cluster_url"]
+        self._ssl_ca_cert = kwargs["ssl_ca_cert"]
+
+        return super().execute_complete(context, event, **kwargs)
