@@ -22,8 +22,10 @@ This module contains Google PubSub operators.
 
     MessageStoragePolicy
 """
+from __future__ import annotations
+
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Sequence
 
 from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
 from google.api_core.retry import Retry
@@ -37,14 +39,15 @@ from google.cloud.pubsub_v1.types import (
     RetryPolicy,
 )
 
-from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.pubsub import PubSubHook
+from airflow.providers.google.cloud.links.pubsub import PubSubSubscriptionLink, PubSubTopicLink
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-class PubSubCreateTopicOperator(BaseOperator):
+class PubSubCreateTopicOperator(GoogleCloudBaseOperator):
     """Create a PubSub topic.
 
     .. seealso::
@@ -91,7 +94,7 @@ class PubSubCreateTopicOperator(BaseOperator):
         of Google Cloud regions where messages published to
         the topic may be stored. If not present, then no constraints
         are in effect.
-        Union[Dict, google.cloud.pubsub_v1.types.MessageStoragePolicy]
+        Union[dict, google.cloud.pubsub_v1.types.MessageStoragePolicy]
     :param kms_key_name: The resource name of the Cloud KMS CryptoKey
         to be used to protect access to messages published on this topic.
         The expected format is
@@ -102,7 +105,6 @@ class PubSubCreateTopicOperator(BaseOperator):
         to complete. Note that if retry is specified, the timeout applies to each
         individual attempt.
     :param metadata: (Optional) Additional metadata that is provided to the method.
-    :param project: (Deprecated) the Google Cloud project ID where the topic will be created
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -114,45 +116,40 @@ class PubSubCreateTopicOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'topic',
-        'impersonation_chain',
+        "project_id",
+        "topic",
+        "impersonation_chain",
     )
-    ui_color = '#0273d4'
+    ui_color = "#0273d4"
+    operator_extra_links = (PubSubTopicLink(),)
 
     def __init__(
         self,
         *,
         topic: str,
-        project_id: Optional[str] = None,
+        project_id: str | None = None,
         fail_if_exists: bool = False,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        labels: Optional[Dict[str, str]] = None,
-        message_storage_policy: Union[Dict, MessageStoragePolicy] = None,
-        kms_key_name: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
-        project: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        labels: dict[str, str] | None = None,
+        message_storage_policy: dict | MessageStoragePolicy = None,
+        kms_key_name: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-        # To preserve backward compatibility
-        # TODO: remove one day
-        if project:
-            warnings.warn(
-                "The project parameter has been deprecated. You should pass the project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            project_id = project
 
         super().__init__(**kwargs)
         self.project_id = project_id
         self.topic = topic
         self.fail_if_exists = fail_if_exists
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.labels = labels
         self.message_storage_policy = message_storage_policy
@@ -162,7 +159,7 @@ class PubSubCreateTopicOperator(BaseOperator):
         self.metadata = metadata
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -182,9 +179,15 @@ class PubSubCreateTopicOperator(BaseOperator):
             metadata=self.metadata,
         )
         self.log.info("Created topic %s", self.topic)
+        PubSubTopicLink.persist(
+            context=context,
+            task_instance=self,
+            topic_id=self.topic,
+            project_id=self.project_id or hook.project_id,
+        )
 
 
-class PubSubCreateSubscriptionOperator(BaseOperator):
+class PubSubCreateSubscriptionOperator(GoogleCloudBaseOperator):
     """Create a PubSub subscription.
 
     .. seealso::
@@ -277,7 +280,7 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
         in which they are received by the Pub/Sub system. Otherwise, they may be
         delivered in any order.
     :param expiration_policy: A policy that specifies the conditions for this
-        subscription’s expiration. A subscription is considered active as long as any
+        subscription's expiration. A subscription is considered active as long as any
         connected subscriber is successfully consuming messages from the subscription or
         is issuing operations on the subscription. If expiration_policy is not set,
         a default policy with ttl of 31 days will be used. The minimum allowed value for
@@ -299,9 +302,6 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
         to complete. Note that if retry is specified, the timeout applies to each
         individual attempt.
     :param metadata: (Optional) Additional metadata that is provided to the method.
-    :param topic_project: (Deprecated) the Google Cloud project ID where the topic exists
-    :param subscription_project: (Deprecated) the Google Cloud project ID where the subscription
-        will be created. If empty, ``topic_project`` will be used.
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -313,62 +313,41 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'topic',
-        'subscription',
-        'subscription_project_id',
-        'impersonation_chain',
+        "project_id",
+        "topic",
+        "subscription",
+        "subscription_project_id",
+        "impersonation_chain",
     )
-    ui_color = '#0273d4'
+    ui_color = "#0273d4"
+    operator_extra_links = (PubSubSubscriptionLink(),)
 
     def __init__(
         self,
         *,
         topic: str,
-        project_id: Optional[str] = None,
-        subscription: Optional[str] = None,
-        subscription_project_id: Optional[str] = None,
+        project_id: str | None = None,
+        subscription: str | None = None,
+        subscription_project_id: str | None = None,
         ack_deadline_secs: int = 10,
         fail_if_exists: bool = False,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        push_config: Optional[Union[Dict, PushConfig]] = None,
-        retain_acked_messages: Optional[bool] = None,
-        message_retention_duration: Optional[Union[Dict, Duration]] = None,
-        labels: Optional[Dict[str, str]] = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        push_config: dict | PushConfig | None = None,
+        retain_acked_messages: bool | None = None,
+        message_retention_duration: dict | Duration | None = None,
+        labels: dict[str, str] | None = None,
         enable_message_ordering: bool = False,
-        expiration_policy: Optional[Union[Dict, ExpirationPolicy]] = None,
-        filter_: Optional[str] = None,
-        dead_letter_policy: Optional[Union[Dict, DeadLetterPolicy]] = None,
-        retry_policy: Optional[Union[Dict, RetryPolicy]] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
-        topic_project: Optional[str] = None,
-        subscription_project: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        expiration_policy: dict | ExpirationPolicy | None = None,
+        filter_: str | None = None,
+        dead_letter_policy: dict | DeadLetterPolicy | None = None,
+        retry_policy: dict | RetryPolicy | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-
-        # To preserve backward compatibility
-        # TODO: remove one day
-        if topic_project:
-            warnings.warn(
-                "The topic_project parameter has been deprecated. You should pass "
-                "the project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            project_id = topic_project
-        if subscription_project:
-            warnings.warn(
-                "The subscription_project parameter has been deprecated. You should pass "
-                "the subscription_project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            subscription_project_id = subscription_project
-
         super().__init__(**kwargs)
         self.project_id = project_id
         self.topic = topic
@@ -377,6 +356,10 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
         self.ack_deadline_secs = ack_deadline_secs
         self.fail_if_exists = fail_if_exists
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.push_config = push_config
         self.retain_acked_messages = retain_acked_messages
@@ -392,7 +375,7 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
         self.metadata = metadata
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> str:
+    def execute(self, context: Context) -> str:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -422,10 +405,16 @@ class PubSubCreateSubscriptionOperator(BaseOperator):
         )
 
         self.log.info("Created subscription for topic %s", self.topic)
+        PubSubSubscriptionLink.persist(
+            context=context,
+            task_instance=self,
+            subscription_id=self.subscription or result,  # result returns subscription name
+            project_id=self.project_id or hook.project_id,
+        )
         return result
 
 
-class PubSubDeleteTopicOperator(BaseOperator):
+class PubSubDeleteTopicOperator(GoogleCloudBaseOperator):
     """Delete a PubSub topic.
 
     .. seealso::
@@ -466,7 +455,6 @@ class PubSubDeleteTopicOperator(BaseOperator):
         to complete. Note that if retry is specified, the timeout applies to each
         individual attempt.
     :param metadata: (Optional) Additional metadata that is provided to the method.
-    :param project: (Deprecated) the Google Cloud project ID where the topic will be created
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -478,49 +466,42 @@ class PubSubDeleteTopicOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'topic',
-        'impersonation_chain',
+        "project_id",
+        "topic",
+        "impersonation_chain",
     )
-    ui_color = '#cb4335'
+    ui_color = "#cb4335"
 
     def __init__(
         self,
         *,
         topic: str,
-        project_id: Optional[str] = None,
+        project_id: str | None = None,
         fail_if_not_exists: bool = False,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
-        project: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-        # To preserve backward compatibility
-        # TODO: remove one day
-        if project:
-            warnings.warn(
-                "The project parameter has been deprecated. You should pass the project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            project_id = project
-
         super().__init__(**kwargs)
         self.project_id = project_id
         self.topic = topic
         self.fail_if_not_exists = fail_if_not_exists
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.retry = retry
         self.timeout = timeout
         self.metadata = metadata
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -539,7 +520,7 @@ class PubSubDeleteTopicOperator(BaseOperator):
         self.log.info("Deleted topic %s", self.topic)
 
 
-class PubSubDeleteSubscriptionOperator(BaseOperator):
+class PubSubDeleteSubscriptionOperator(GoogleCloudBaseOperator):
     """Delete a PubSub subscription.
 
     .. seealso::
@@ -582,7 +563,6 @@ class PubSubDeleteSubscriptionOperator(BaseOperator):
         to complete. Note that if retry is specified, the timeout applies to each
         individual attempt.
     :param metadata: (Optional) Additional metadata that is provided to the method.
-    :param project: (Deprecated) the Google Cloud project ID where the topic will be created
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -594,49 +574,42 @@ class PubSubDeleteSubscriptionOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'subscription',
-        'impersonation_chain',
+        "project_id",
+        "subscription",
+        "impersonation_chain",
     )
-    ui_color = '#cb4335'
+    ui_color = "#cb4335"
 
     def __init__(
         self,
         *,
         subscription: str,
-        project_id: Optional[str] = None,
+        project_id: str | None = None,
         fail_if_not_exists: bool = False,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
-        project: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-        # To preserve backward compatibility
-        # TODO: remove one day
-        if project:
-            warnings.warn(
-                "The project parameter has been deprecated. You should pass the project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            project_id = project
-
         super().__init__(**kwargs)
         self.project_id = project_id
         self.subscription = subscription
         self.fail_if_not_exists = fail_if_not_exists
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.retry = retry
         self.timeout = timeout
         self.metadata = metadata
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -655,7 +628,7 @@ class PubSubDeleteSubscriptionOperator(BaseOperator):
         self.log.info("Deleted subscription %s", self.subscription)
 
 
-class PubSubPublishMessageOperator(BaseOperator):
+class PubSubPublishMessageOperator(GoogleCloudBaseOperator):
     """Publish messages to a PubSub topic.
 
     .. seealso::
@@ -702,7 +675,6 @@ class PubSubPublishMessageOperator(BaseOperator):
     :param delegate_to: The account to impersonate using domain-wide delegation of authority,
         if any. For this to work, the service account making the request must have
         domain-wide delegation enabled.
-    :param project: (Deprecated) the Google Cloud project ID where the topic will be created
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -714,44 +686,37 @@ class PubSubPublishMessageOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'topic',
-        'messages',
-        'impersonation_chain',
+        "project_id",
+        "topic",
+        "messages",
+        "impersonation_chain",
     )
-    ui_color = '#0273d4'
+    ui_color = "#0273d4"
 
     def __init__(
         self,
         *,
         topic: str,
-        messages: List,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        project: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        messages: list,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
-        # To preserve backward compatibility
-        # TODO: remove one day
-        if project:
-            warnings.warn(
-                "The project parameter has been deprecated. You should pass the project_id parameter.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-            project_id = project
-
         super().__init__(**kwargs)
         self.project_id = project_id
         self.topic = topic
         self.messages = messages
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -763,7 +728,7 @@ class PubSubPublishMessageOperator(BaseOperator):
         self.log.info("Published to topic %s", self.topic)
 
 
-class PubSubPullOperator(BaseOperator):
+class PubSubPullOperator(GoogleCloudBaseOperator):
     """Pulls messages from a PubSub subscription and passes them through XCom.
     If the queue is empty, returns empty list - never waits for messages.
     If you do need to wait, please use :class:`airflow.providers.google.cloud.sensors.PubSubPullSensor`
@@ -811,9 +776,9 @@ class PubSubPullOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'project_id',
-        'subscription',
-        'impersonation_chain',
+        "project_id",
+        "subscription",
+        "impersonation_chain",
     )
 
     def __init__(
@@ -823,14 +788,18 @@ class PubSubPullOperator(BaseOperator):
         subscription: str,
         max_messages: int = 5,
         ack_messages: bool = False,
-        messages_callback: Optional[Callable[[List[ReceivedMessage], "Context"], Any]] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        messages_callback: Callable[[list[ReceivedMessage], Context], Any] | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.project_id = project_id
         self.subscription = subscription
@@ -839,7 +808,7 @@ class PubSubPullOperator(BaseOperator):
         self.messages_callback = messages_callback
         self.impersonation_chain = impersonation_chain
 
-    def execute(self, context: 'Context') -> list:
+    def execute(self, context: Context) -> list:
         hook = PubSubHook(
             gcp_conn_id=self.gcp_conn_id,
             delegate_to=self.delegate_to,
@@ -868,8 +837,8 @@ class PubSubPullOperator(BaseOperator):
 
     def _default_message_callback(
         self,
-        pulled_messages: List[ReceivedMessage],
-        context: "Context",
+        pulled_messages: list[ReceivedMessage],
+        context: Context,
     ) -> list:
         """
         This method can be overridden by subclasses or by `messages_callback` constructor argument.
