@@ -26,15 +26,15 @@ from google.cloud.speech_v1.types import RecognitionConfig
 from google.protobuf.json_format import MessageToDict
 
 from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.speech_to_text import CloudSpeechToTextHook, RecognitionAudio
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 from airflow.providers.google.common.links.storage import FileDetailsLink
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-class CloudSpeechToTextRecognizeSpeechOperator(BaseOperator):
+class CloudSpeechToTextRecognizeSpeechOperator(GoogleCloudBaseOperator):
     """
     Recognizes speech from audio file and returns it as text.
 
