@@ -16,14 +16,16 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains a Google Cloud Storage operator."""
+from __future__ import annotations
+
 import warnings
-from typing import TYPE_CHECKING, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Sequence
 
 from airflow.exceptions import AirflowException
 from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.gcs import GCSHook
 
-WILDCARD = '*'
+WILDCARD = "*"
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
@@ -69,8 +71,6 @@ class GCSToGCSOperator(BaseOperator):
         If source_objects = ['foo/bah/'] and delimiter = '.avro', then only the 'files' in the
         folder 'foo/bah/' with '.avro' delimiter will be copied to the destination object.
     :param gcp_conn_id: (Optional) The connection ID used to connect to Google Cloud.
-    :param google_cloud_storage_conn_id: (Deprecated) The connection ID used to connect to Google Cloud.
-        This parameter has been deprecated. You should pass the gcp_conn_id parameter instead.
     :param delegate_to: The account to impersonate using domain-wide delegation of authority,
         if any. For this to work, the service account making the request must have
         domain-wide delegation enabled.
@@ -92,6 +92,8 @@ class GCSToGCSOperator(BaseOperator):
         account from the list granting this role to the originating account (templated).
     :param source_object_required: Whether you want to raise an exception when the source object
         doesn't exist. It doesn't have any effect when the source objects are folders or patterns.
+    :param exact_match: When specified, only exact match of the source object (filename) will be
+        copied.
 
     :Example:
 
@@ -148,7 +150,7 @@ class GCSToGCSOperator(BaseOperator):
         )
 
     The following Operator would move all the Avro files from ``sales/sales-2019``
-     and ``sales/sales-2020` folder in ``data`` bucket to the same folder in the
+     and ``sales/sales-2020`` folder in ``data`` bucket to the same folder in the
      ``data_backup`` bucket, deleting the original files in the process. ::
 
         move_files = GCSToGCSOperator(
@@ -164,15 +166,15 @@ class GCSToGCSOperator(BaseOperator):
     """
 
     template_fields: Sequence[str] = (
-        'source_bucket',
-        'source_object',
-        'source_objects',
-        'destination_bucket',
-        'destination_object',
-        'delimiter',
-        'impersonation_chain',
+        "source_bucket",
+        "source_object",
+        "source_objects",
+        "destination_bucket",
+        "destination_object",
+        "delimiter",
+        "impersonation_chain",
     )
-    ui_color = '#f0eee4'
+    ui_color = "#f0eee4"
 
     def __init__(
         self,
@@ -185,25 +187,17 @@ class GCSToGCSOperator(BaseOperator):
         delimiter=None,
         move_object=False,
         replace=True,
-        gcp_conn_id='google_cloud_default',
-        google_cloud_storage_conn_id=None,
+        gcp_conn_id="google_cloud_default",
         delegate_to=None,
         last_modified_time=None,
         maximum_modified_time=None,
         is_older_than=None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         source_object_required=False,
+        exact_match=False,
         **kwargs,
     ):
         super().__init__(**kwargs)
-        if google_cloud_storage_conn_id:
-            warnings.warn(
-                "The google_cloud_storage_conn_id parameter has been deprecated. You should pass "
-                "the gcp_conn_id parameter.",
-                DeprecationWarning,
-                stacklevel=3,
-            )
-            gcp_conn_id = google_cloud_storage_conn_id
 
         self.source_bucket = source_bucket
         self.source_object = source_object
@@ -214,14 +208,19 @@ class GCSToGCSOperator(BaseOperator):
         self.move_object = move_object
         self.replace = replace
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.last_modified_time = last_modified_time
         self.maximum_modified_time = maximum_modified_time
         self.is_older_than = is_older_than
         self.impersonation_chain = impersonation_chain
         self.source_object_required = source_object_required
+        self.exact_match = exact_match
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
 
         hook = GCSHook(
             gcp_conn_id=self.gcp_conn_id,
@@ -240,7 +239,7 @@ class GCSToGCSOperator(BaseOperator):
             raise AirflowException(error_msg)
 
         if self.source_objects and not all(isinstance(item, str) for item in self.source_objects):
-            raise AirflowException('At least, one of the `objects` in the `source_objects` is not a string')
+            raise AirflowException("At least, one of the `objects` in the `source_objects` is not a string")
 
         # If source_object is set, default it to source_objects
         if self.source_object:
@@ -248,15 +247,15 @@ class GCSToGCSOperator(BaseOperator):
 
         if self.destination_bucket is None:
             self.log.warning(
-                'destination_bucket is None. Defaulting it to source_bucket (%s)', self.source_bucket
+                "destination_bucket is None. Defaulting it to source_bucket (%s)", self.source_bucket
             )
             self.destination_bucket = self.source_bucket
 
         # An empty source_object means to copy all files
         if len(self.source_objects) == 0:
-            self.source_objects = ['']
+            self.source_objects = [""]
         # Raise exception if empty string `''` is used twice in source_object, this is to avoid double copy
-        if self.source_objects.count('') > 1:
+        if self.source_objects.count("") > 1:
             raise AirflowException("You can't have two empty strings inside source_object")
 
         # Iterate over the source_objects and do the copy
@@ -268,6 +267,32 @@ class GCSToGCSOperator(BaseOperator):
             else:
                 self._copy_source_without_wildcard(hook=hook, prefix=prefix)
 
+    def _ignore_existing_files(self, hook, prefix, **kwargs):
+        # list all files in the Destination GCS bucket
+        # and only keep those files which are present in
+        # Source GCS bucket and not in Destination GCS bucket
+        delimiter = kwargs.get("delimiter")
+        objects = kwargs.get("objects")
+        if self.destination_object is None:
+            existing_objects = hook.list(self.destination_bucket, prefix=prefix, delimiter=delimiter)
+        else:
+            self.log.info("Replaced destination_object with source_object prefix.")
+            destination_objects = hook.list(
+                self.destination_bucket,
+                prefix=self.destination_object,
+                delimiter=delimiter,
+            )
+            existing_objects = [
+                dest_object.replace(self.destination_object, prefix, 1) for dest_object in destination_objects
+            ]
+
+        objects = set(objects) - set(existing_objects)
+        if len(objects) > 0:
+            self.log.info("%s files are going to be synced: %s.", len(objects), objects)
+        else:
+            self.log.info("There are no new files to sync. Have a nice day!")
+        return objects
+
     def _copy_source_without_wildcard(self, hook, prefix):
         """
         For source_objects with no wildcard, this operator would first list
@@ -278,8 +303,8 @@ class GCSToGCSOperator(BaseOperator):
         Example 1:
 
 
-        The following Operator would copy all the files from ``a/``folder
-        (i.e a/a.csv, a/b.csv, a/c.csv)in ``data`` bucket to the ``b/`` folder in
+        The following Operator would copy all the files from ``a/`` folder
+        (i.e a/a.csv, a/b.csv, a/c.csv) in ``data`` bucket to the ``b/`` folder in
         the ``data_backup`` bucket (b/a.csv, b/b.csv, b/c.csv) ::
 
             copy_files = GCSToGCSOperator(
@@ -294,8 +319,8 @@ class GCSToGCSOperator(BaseOperator):
         Example 2:
 
 
-        The following Operator would copy all avro files from ``a/``folder
-        (i.e a/a.avro, a/b.avro, a/c.avro)in ``data`` bucket to the ``b/`` folder in
+        The following Operator would copy all avro files from ``a/`` folder
+        (i.e a/a.avro, a/b.avro, a/c.avro) in ``data`` bucket to the ``b/`` folder in
         the ``data_backup`` bucket (b/a.avro, b/b.avro, b/c.avro) ::
 
             copy_files = GCSToGCSOperator(
@@ -307,10 +332,30 @@ class GCSToGCSOperator(BaseOperator):
                 delimiter='.avro',
                 gcp_conn_id=google_cloud_conn_id
             )
+
+        Example 3:
+
+
+        The following Operator would copy files (a/file_1.txt, a/file_2.csv, a/file_3.avro)
+        in ``data`` bucket to the ``b/`` folder in
+        the ``data_backup`` bucket (b/file_1.txt, b/file_2.csv, b/file_3.avro) ::
+
+            copy_files = GCSToGCSOperator(
+                task_id='copy_files_without_wildcard',
+                source_bucket='data',
+                source_objects=['a/file_1.txt', 'a/file_2.csv', 'a/file_3.avro'],
+                destination_bucket='data_backup',
+                destination_object='b/',
+                gcp_conn_id=google_cloud_conn_id
+            )
         """
         objects = hook.list(self.source_bucket, prefix=prefix, delimiter=self.delimiter)
 
-        # If objects is empty and we have prefix, let's check if prefix is a blob
+        if not self.replace:
+            # If we are not replacing, ignore files already existing in source buckets
+            objects = self._ignore_existing_files(hook, prefix, objects=objects, delimiter=self.delimiter)
+
+        # If objects is empty, and we have prefix, let's check if prefix is a blob
         # and copy directly
         if len(objects) == 0 and prefix:
             if hook.exists(self.source_bucket, prefix):
@@ -322,11 +367,31 @@ class GCSToGCSOperator(BaseOperator):
                 self.log.warning(msg)
                 raise AirflowException(msg)
 
-        for source_obj in objects:
+        if len(objects) == 1 and objects[0][-1] != "/":
+            self._copy_file(hook=hook, source_object=objects[0])
+        elif len(objects):
+            self._copy_directory(hook=hook, source_objects=objects, prefix=prefix)
+
+    def _copy_file(self, hook, source_object):
+        destination_object = self.destination_object or source_object
+        if self.destination_object and self.destination_object[-1] == "/":
+            file_name = source_object.split("/")[-1]
+            destination_object += file_name
+        self._copy_single_object(
+            hook=hook, source_object=source_object, destination_object=destination_object
+        )
+
+    def _copy_directory(self, hook, source_objects, prefix):
+        _prefix = prefix.rstrip("/") + "/"
+        for source_obj in source_objects:
+            if self.exact_match and (source_obj != prefix or not source_obj.endswith(prefix)):
+                continue
             if self.destination_object is None:
                 destination_object = source_obj
             else:
-                destination_object = source_obj.replace(prefix, self.destination_object, 1)
+                file_name_postfix = source_obj.replace(_prefix, "", 1)
+                destination_object = self.destination_object.rstrip("/") + "/" + file_name_postfix
+
             self._copy_single_object(
                 hook=hook, source_object=source_obj, destination_object=destination_object
             )
@@ -340,33 +405,15 @@ class GCSToGCSOperator(BaseOperator):
             )
 
             raise AirflowException(error_msg)
-        self.log.info('Delimiter ignored because wildcard is in prefix')
+        self.log.info("Delimiter ignored because wildcard is in prefix")
         prefix_, delimiter = prefix.split(WILDCARD, 1)
         objects = hook.list(self.source_bucket, prefix=prefix_, delimiter=delimiter)
         if not self.replace:
             # If we are not replacing, list all files in the Destination GCS bucket
             # and only keep those files which are present in
             # Source GCS bucket and not in Destination GCS bucket
+            objects = self._ignore_existing_files(hook, prefix_, delimiter=delimiter, objects=objects)
 
-            if self.destination_object is None:
-                existing_objects = hook.list(self.destination_bucket, prefix=prefix_, delimiter=delimiter)
-            else:
-                self.log.info("Replaced destination_object with source_object prefix.")
-                destination_objects = hook.list(
-                    self.destination_bucket,
-                    prefix=self.destination_object,
-                    delimiter=delimiter,
-                )
-                existing_objects = [
-                    dest_object.replace(self.destination_object, prefix_, 1)
-                    for dest_object in destination_objects
-                ]
-
-            objects = set(objects) - set(existing_objects)
-            if len(objects) > 0:
-                self.log.info('%s files are going to be synced: %s.', len(objects), objects)
-            else:
-                self.log.info('There are no new files to sync. Have a nice day!')
         for source_object in objects:
             if self.destination_object is None:
                 destination_object = source_object
@@ -421,7 +468,7 @@ class GCSToGCSOperator(BaseOperator):
                 return
 
         self.log.info(
-            'Executing copy of gs://%s/%s to gs://%s/%s',
+            "Executing copy of gs://%s/%s to gs://%s/%s",
             self.source_bucket,
             source_object,
             self.destination_bucket,
