@@ -15,23 +15,33 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import os
 import shutil
-import sys
-from typing import Dict, Optional, Tuple
+from pathlib import Path
+from typing import TYPE_CHECKING, Any
 
-from azure.common import AzureHttpError
-
-if sys.version_info >= (3, 8):
-    from functools import cached_property
-else:
-    from cached_property import cached_property
+from azure.core.exceptions import HttpResponseError
+from packaging.version import Version
 
+from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.utils.log.file_task_handler import FileTaskHandler
 from airflow.utils.log.logging_mixin import LoggingMixin
 
 
+def get_default_delete_local_copy():
+    """Load delete_local_logs conf if Airflow version > 2.6 and return False if not
+    TODO: delete this function when min airflow version >= 2.6
+    """
+    from airflow.version import version
+
+    if Version(version) < Version("2.6"):
+        return False
+    return conf.getboolean("logging", "delete_local_logs")
+
+
 class WasbTaskHandler(FileTaskHandler, LoggingMixin):
     """
     WasbTaskHandler is a python log handler that handles and reads
@@ -39,37 +49,42 @@ class WasbTaskHandler(FileTaskHandler, LoggingMixin):
     uploads to and reads from Wasb remote storage.
     """
 
+    trigger_should_wrap = True
+
     def __init__(
         self,
         base_log_folder: str,
         wasb_log_folder: str,
         wasb_container: str,
-        delete_local_copy: str,
         *,
-        filename_template: Optional[str] = None,
+        filename_template: str | None = None,
+        **kwargs,
     ) -> None:
         super().__init__(base_log_folder, filename_template)
         self.wasb_container = wasb_container
         self.remote_base = wasb_log_folder
-        self.log_relative_path = ''
+        self.log_relative_path = ""
         self._hook = None
         self.closed = False
         self.upload_on_close = True
-        self.delete_local_copy = delete_local_copy
+        self.delete_local_copy = (
+            kwargs["delete_local_copy"] if "delete_local_copy" in kwargs else get_default_delete_local_copy()
+        )
 
     @cached_property
     def hook(self):
         """Returns WasbHook."""
-        remote_conn_id = conf.get('logging', 'REMOTE_LOG_CONN_ID')
+        remote_conn_id = conf.get("logging", "REMOTE_LOG_CONN_ID")
         try:
             from airflow.providers.microsoft.azure.hooks.wasb import WasbHook
 
             return WasbHook(remote_conn_id)
-        except AzureHttpError:
+        except Exception:
             self.log.exception(
-                'Could not create an WasbHook with connection id "%s".'
-                ' Please make sure that apache-airflow[azure] is installed'
-                ' and the Wasb connection exists.',
+                "Could not create a WasbHook with connection id '%s'. "
+                "Do you have apache-airflow[azure] installed? "
+                "Does connection the connection exist, and is it "
+                "configured properly?",
                 remote_conn_id,
             )
             return None
@@ -78,8 +93,13 @@ class WasbTaskHandler(FileTaskHandler, LoggingMixin):
         super().set_context(ti)
         # Local location and remote location is needed to open and
         # upload local log file to Wasb remote storage.
-        self.log_relative_path = self._render_filename(ti, ti.try_number)
-        self.upload_on_close = not ti.raw
+        if TYPE_CHECKING:
+            assert self.handler is not None
+
+        full_path = self.handler.baseFilename
+        self.log_relative_path = Path(full_path).relative_to(self.local_base).as_posix()
+        is_trigger_log_context = getattr(ti, "is_trigger_log_context", False)
+        self.upload_on_close = is_trigger_log_context or not ti.raw
 
     def close(self) -> None:
         """Close and upload local log file to remote storage Wasb."""
@@ -101,38 +121,74 @@ class WasbTaskHandler(FileTaskHandler, LoggingMixin):
             # read log and remove old logs to get just the latest additions
             with open(local_loc) as logfile:
                 log = logfile.read()
-            self.wasb_write(log, remote_loc, append=True)
+            wasb_write = self.wasb_write(log, remote_loc, append=True)
 
-            if self.delete_local_copy:
+            if wasb_write and self.delete_local_copy:
                 shutil.rmtree(os.path.dirname(local_loc))
         # Mark closed so we don't double write if close is called twice
         self.closed = True
 
-    def _read(self, ti, try_number: int, metadata: Optional[str] = None) -> Tuple[str, Dict[str, bool]]:
+    def _read_remote_logs(self, ti, try_number, metadata=None):
+        messages = []
+        logs = []
+        worker_log_relative_path = self._render_filename(ti, try_number)
+        # todo: fix this
+        # for some reason this handler was designed such that (1) container name is not configurable
+        # (i.e. it's hardcoded in airflow_local_settings.py) and (2) the "relative path" is actually...
+        # whatever you put in REMOTE_BASE_LOG_FOLDER i.e. it includes the "wasb://" in the blob
+        # name. it's very screwed up but to change it we have to be careful not to break backcompat.
+        prefix = os.path.join(self.remote_base, worker_log_relative_path)
+        blob_names = []
+        try:
+            blob_names = self.hook.get_blobs_list(container_name=self.wasb_container, prefix=prefix)
+        except HttpResponseError as e:
+            messages.append(f"tried listing blobs with prefix={prefix} and container={self.wasb_container}")
+            messages.append("could not list blobs " + str(e))
+            self.log.exception("can't list blobs")
+
+        if blob_names:
+            uris = [f"wasb://{self.wasb_container}/{b}" for b in blob_names]
+            messages.extend(["Found remote logs:", *[f"  * {x}" for x in sorted(uris)]])
+        else:
+            messages.append(f"No logs found in WASB; ti=%s {ti}")
+
+        for name in sorted(blob_names):
+            remote_log = ""
+            try:
+                remote_log = self.hook.read_file(self.wasb_container, name)
+                if remote_log:
+                    logs.append(remote_log)
+            except Exception as e:
+                messages.append(
+                    f"Unable to read remote blob '{name}' in container '{self.wasb_container}'\n{e}"
+                )
+                self.log.exception("Could not read blob")
+        return messages, logs
+
+    def _read(
+        self, ti, try_number: int, metadata: dict[str, Any] | None = None
+    ) -> tuple[str, dict[str, bool]]:
         """
         Read logs of given task instance and try_number from Wasb remote storage.
         If failed, read the log from task instance host machine.
 
+        todo: when min airflow version >= 2.6, remove this method
+
         :param ti: task instance object
         :param try_number: task instance try_number to read logs from
         :param metadata: log metadata,
                          can be used for steaming log reading and auto-tailing.
         """
-        # Explicitly getting log relative path is necessary as the given
-        # task instance might be different than task instance passed in
-        # in set_context method.
-        log_relative_path = self._render_filename(ti, try_number)
-        remote_loc = os.path.join(self.remote_base, log_relative_path)
+        if hasattr(super(), "_read_remote_logs"):
+            # from Airflow 2.6, we don't implement the `_read` method.
+            # if parent has _read_remote_logs, we're >= 2.6
+            return super()._read(ti, try_number, metadata)
 
-        if self.wasb_log_exists(remote_loc):
-            # If Wasb remote file exists, we do not fetch logs from task instance
-            # local machine even if there are errors reading remote logs, as
-            # returned remote_log will contain error messages.
-            remote_log = self.wasb_read(remote_loc, return_error=True)
-            log = f'*** Reading remote log from {remote_loc}.\n{remote_log}\n'
-            return log, {'end_of_log': True}
-        else:
-            return super()._read(ti, try_number)
+        # below is backcompat, for airflow < 2.6
+        messages, logs = self._read_remote_logs(ti, try_number, metadata)
+        if not logs:
+            return super()._read(ti, try_number, metadata)
+        return "".join([f"*** {x}\n" for x in messages]) + "\n".join(logs), {"end_of_log": True}
 
     def wasb_log_exists(self, remote_log_location: str) -> bool:
         """
@@ -159,15 +215,15 @@ class WasbTaskHandler(FileTaskHandler, LoggingMixin):
         """
         try:
             return self.hook.read_file(self.wasb_container, remote_log_location)
-        except AzureHttpError:
-            msg = f'Could not read logs from {remote_log_location}'
+        except Exception:
+            msg = f"Could not read logs from {remote_log_location}"
             self.log.exception(msg)
             # return error if needed
             if return_error:
                 return msg
-            return ''
+            return ""
 
-    def wasb_write(self, log: str, remote_log_location: str, append: bool = True) -> None:
+    def wasb_write(self, log: str, remote_log_location: str, append: bool = True) -> bool:
         """
         Writes the log to the remote_log_location. Fails silently if no hook
         was created.
@@ -179,9 +235,11 @@ class WasbTaskHandler(FileTaskHandler, LoggingMixin):
         """
         if append and self.wasb_log_exists(remote_log_location):
             old_log = self.wasb_read(remote_log_location)
-            log = '\n'.join([old_log, log]) if old_log else log
+            log = "\n".join([old_log, log]) if old_log else log
 
         try:
             self.hook.load_string(log, self.wasb_container, remote_log_location, overwrite=True)
-        except AzureHttpError:
-            self.log.exception('Could not write logs to %s', remote_log_location)
+        except Exception:
+            self.log.exception("Could not write logs to %s", remote_log_location)
+            return False
+        return True
