@@ -51,3 +51,7 @@ class DagAttributeTypes(str, Enum):
     XCOM_REF = "xcomref"
     DATASET = "dataset"
     SIMPLE_TASK_INSTANCE = "simple_task_instance"
+    BASE_JOB = "Job"
+    TASK_INSTANCE = "task_instance"
+    DAG_RUN = "dag_run"
+    DATA_SET = "data_set"
